CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
114 lines
3.4 KiB
C++
114 lines
3.4 KiB
C++
/* **************************************************************************
|
|
* Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
|
|
*
|
|
* Redistribution and use in source and binary forms, with or without
|
|
* modification, are permitted provided that the following conditions
|
|
* are met:
|
|
*
|
|
* 1. Redistributions of source code must retain the above copyright
|
|
* notice, this list of conditions and the following disclaimer.
|
|
*
|
|
* 2. Redistributions in binary form must reproduce the above copyright
|
|
* notice, this list of conditions and the following disclaimer in the
|
|
* documentation and/or other materials provided with the distribution.
|
|
*
|
|
* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
|
|
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
|
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
|
|
* ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
|
|
* FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
|
|
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
|
|
* OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
|
|
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
|
|
* LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
|
|
* OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
|
|
* SUCH DAMAGE.
|
|
* *************************************************************************/
|
|
|
|
#pragma once
|
|
|
|
#include <cmath>
|
|
#include <hip/hip_runtime.h>
|
|
#include <rocblas/rocblas.h>
|
|
#include <type_traits>
|
|
|
|
/* ============================================================================================
|
|
*/
|
|
// Helper function to truncate float to bfloat16
|
|
|
|
inline __host__ rocblas_bfloat16 float_to_bfloat16_truncate(float val)
|
|
{
|
|
union
|
|
{
|
|
float fp32;
|
|
uint32_t int32;
|
|
} u = {val};
|
|
rocblas_bfloat16 ret;
|
|
ret.data = uint16_t(u.int32 >> 16);
|
|
if((u.int32 & 0x7fff0000) == 0x7f800000 && u.int32 & 0xffff)
|
|
ret.data |= 1; // Preserve signaling NaN
|
|
return ret;
|
|
}
|
|
|
|
/* ============================================================================================
|
|
*/
|
|
/*! \brief returns true if value is NaN */
|
|
|
|
template <typename T, std::enable_if_t<std::is_integral<T>{}, int> = 0>
|
|
inline bool rocblas_isnan(T)
|
|
{
|
|
return false;
|
|
}
|
|
|
|
template <typename T, std::enable_if_t<!std::is_integral<T>{} && !rocblas_is_complex<T>, int> = 0>
|
|
inline bool rocblas_isnan(T arg)
|
|
{
|
|
return std::isnan(arg);
|
|
}
|
|
|
|
template <typename T, std::enable_if_t<rocblas_is_complex<T>, int> = 0>
|
|
inline bool rocblas_isnan(const T& arg)
|
|
{
|
|
return rocblas_isnan(std::real(arg)) || rocblas_isnan(std::imag(arg));
|
|
}
|
|
|
|
inline bool rocblas_isnan(rocblas_half arg)
|
|
{
|
|
union
|
|
{
|
|
rocblas_half fp;
|
|
uint16_t data;
|
|
} x = {arg};
|
|
return (~x.data & 0x7c00) == 0 && (x.data & 0x3ff) != 0;
|
|
}
|
|
|
|
/* ============================================================================================
|
|
*/
|
|
/*! \brief negate a value */
|
|
|
|
template <class T>
|
|
inline T negate(T x)
|
|
{
|
|
return -x;
|
|
}
|
|
|
|
template <>
|
|
inline rocblas_half negate(rocblas_half arg)
|
|
{
|
|
union
|
|
{
|
|
rocblas_half fp;
|
|
uint16_t data;
|
|
} x = {arg};
|
|
|
|
x.data ^= 0x8000;
|
|
return x.fp;
|
|
}
|
|
|
|
template <>
|
|
inline rocblas_bfloat16 negate(rocblas_bfloat16 x)
|
|
{
|
|
x.data ^= 0x8000;
|
|
return x;
|
|
}
|